<?php

namespace Drupal\recurly_aegir\Processors;

use Symfony\Component\HttpFoundation\Request;
use Drupal\Component\Utility\Html;
use Drupal\Core\Url;
use Drupal\Core\Link;
use Drupal\node\Entity\Node;
use Drupal\recurly_aegir\Provisioners\RecurlyAegirSiteProvisioner;

/**
 * Processes Recurly notifications for Aegir.
 *
 * @todo Make this class and all subclasses services?
 */
abstract class RecurlyAegirProcessor {

  /**
   * @var SimpleXMLElement|NULL $account
   *   Account information.
   */
  protected $account;

  /**
   * @var SimpleXMLElement|NULL $subscription
   *   Subscription information.
   */
  protected $subscription;

  /**
   * @var SimpleXMLElement|NULL $transaction
   *   Transaction information.
   */
  protected $transaction;

  /**
   * @var SimpleXMLElement|NULL $invoice
   *   Invoice information.
   */
  protected $invoice;

  /**
   * @var boolean $result
   *   TRUE if processing was successful; FALSE otherwise.
   */
  protected $result = FALSE;

  /**
   * Class constuctor.
   *
   * @param SimpleXMLElement|NULL $account
   *   Account information.
   * @param SimpleXMLElement|NULL $subscription
   *   Subscription information.
   * @param SimpleXMLElement|NULL $transaction
   *   Transaction information.
   * @param SimpleXMLElement|NULL $invoice
   *   Invoice information.
   * @return $this
   *   The newly created object.
   */
  public function __construct(
      $account = NULL,
      $subscription = NULL,
      $transaction = NULL,
      $invoice = NULL) {
    $this->account = $account;
    $this->subscription = $subscription;
    $this->transaction = $transaction;
    $this->invoice = $invoice;
  }

  /**
   * Process any notifications coming from Recurly.
   *
   * Must be overriden by subclasses to actually do something.
   *
   * @return $this
   *   The object itself.
   */
  abstract public function process();

  /**
   * Get the processing result.
   *
   * @return boolean
   *   TRUE if processing was successful; FALSE otherwise.
   */
  public function getResult() {
    return $this->result;
  }

  /**
   * Gets a user ID from an account code.
   *
   * Given a Recurly account code, returns a local Drupal user ID.
   *
   * @param SimpleXMLElement $account
   *   The user's account code within Recurly.
   * @return integer
   *   The local Drupal user ID.
   *
   * @todo Add support for account entities other than User.
   * @todo Move this into the base Recurly module.
   */
  protected function getUserId(\SimpleXMLElement $account) {
    $delimiter = '-';

    // Throw away the first part of the code as we only care about the ID.
    strtok((string) $account->account_code, $delimiter);

    // Grab the second part as the user ID.
    $user_id = strtok($delimiter);

    // Compare the internal username with the one provided by Recurly.
    $user = \Drupal\user\Entity\User::load($user_id);
    if ($user->get('name')->value != (string) $account->username) {
      $message = "User ID %userid: Drupal username %drupal does not match Recurly username %recurly.";
      \Drupal::logger('recurly_aegir')->warning($message, [
        '%userid' => $user_id,
        '%drupal' => $user->get('name')->value,
        '%recurly' => (string) $account->username,
      ]);
    }

    // Compare the internal e-mail address with the one provided by Recurly.
    if ($user->get('mail')->value != (string) $account->email) {
      $message = "User ID %userid: Drupal e-mail %drupal does not match Recurly e-mail %recurly.";
      \Drupal::logger('recurly_aegir')->warning($message, [
        '%userid' => $user_id,
        '%drupal' => $user->get('mail')->value,
        '%recurly' => (string) $account->email,
      ]);
    }

    // Return it.
    return $user_id;
  }

  /**
   * Gets the Recurly subscription URL from a UUID.
   *
   * @param string $uuid
   * @return string
   *   The subscription URL on the Recurly subdomain.
   */
  public static function getSubscriptionUrl($uuid) {
    $subdomain = \Drupal::config('recurly.settings')->get('recurly_subdomain');
    return 'https://' . $subdomain . '.recurly.com/subscriptions/' . $uuid;
  }

  /**
   * Gets a site's subscription ID.
   *
   * @param Node $site
   *   The site whose ID is being requested.
   * @return string
   *   The subscription's UUID.
   */
  public static function getSubscriptionId(Node $site) {
    return $site->get('field_site_subscription_url')->getValue()[0]['title'];
  }

  /**
   * Gets the configuration link for a specific site.
   *
   * @param Node $site
   *   The site node.
   * @return Link
   *   The site node's configuration link.
   */
  public static function getSiteConfigurationLink(Node $site) {
    $url = Url::fromRoute('entity.node.edit_form', ['node' => $site->id()]);
    return Link::fromTextAndUrl(t('<em>Configuration required!</em>'), $url);
  }

  /**
   * Gets the link for a specific client site.
   *
   * @param Node $site
   *   The site node.
   * @return Link
   *   A link to the site itself.
   */
  public static function getSiteLink(Node $site) {
    $protocol = \Drupal::request()->getScheme();
    $subdomain = Html::escape($site->getTitle());
    $domain = \Drupal::request()->getHost();
    $hostname = "$subdomain.$domain";
    $url = Url::fromUri("$protocol://$hostname/");
    return Link::fromTextAndUrl($hostname, $url);
  }

  /**
   * Returns the endpoint for the Aegir Web service.
   *
   * @return string
   *   The endpoint URL.
   */
  public static function getServiceEndpoint() {
    $endpoint = \Drupal::config('recurly_aegir.settings')->get('service_endpoint_url');
    if (empty($endpoint)) {
      \Drupal::logger('recurly_aegir')
        ->critical('Cannot run processor: Service endpoint has not been configured.');
    }
    return $endpoint;
  }

  /**
   * Launch a provisioner for a specific site.
   *
   * @param Node $site
   *   The site to be provisioned.
   * @param RecurlyAegirSiteProvisioner $provisioner
   *   The provisioner doing the provisioning.
   */
  public static function launchSiteProvisioner(Node $site, RecurlyAegirSiteProvisioner $provisioner) {
    try {
      // Provision the new site and get the task ID.
      $task_id = $provisioner->provision()->getTaskId();

      // Save a link to the task ID for future reference.
      $aegir_url = Request::create($provisioner->getEndpoint())->getSchemeAndHttpHost();
      $site->field_site_tasks->appendItem([
        'uri' => $aegir_url . '/hosting/task/' . $task_id,
        'title' => date("Y-m-d \@ H:i:s") . ' » ' . $provisioner->getTaskType(),
      ]);

      // Log the successful provisioning task creation along with the task ID.
      $template = $provisioner->getTemplate();
      \Drupal::logger('recurly_aegir')
        ->info('Task %task created to provision %hostname from %template via %provisioner.', [
          '%task' => $task_id,
          '%hostname' => $provisioner->getHostname(),
          '%template' => is_null($template) ? 'NO TEMPLATE' : $template,
          '%provisioner' => (new \ReflectionClass($provisioner))->getShortName(),
        ]);
    }
    catch (RecurlyAegirTaskCreationFailedException $e) {
      watchdog_exception('recurly_aegir', $e);
    }
  }

  /**
   * Determines if a particular subscription is active for a site.
   *
   * @param Node $site
   *   The site in question.
   * @return boolean
   *   TRUE if subscription is active; FALSE otherwise.
   */
  public static function subscriptionIsActive(Node $site) {
    if (recurly_client_initialize()) {
      $uuid = self::getSubscriptionId($site);
      $state = \Recurly_Subscription::get($uuid)->state;
      if ($state == 'active') {
        return TRUE;
      }
    }
    return FALSE;
  }

}
