<?php

namespace Drupal\recurly_aegir\Provisioners;

use Drupal\recurly_aegir\Exceptions\RecurlyAegirTaskCreationFailedException;

/**
 * Base class for provisioning new sites via Aegir's Web service API.
 *
 * @todo Make this class a plugin manager and make the subclasses plugins.
 */
abstract class RecurlyAegirSiteProvisioner {

  /**
   * The list of provisioner classes available for creating new sites.
   *
   * This won't be necessary when we switch to plug-in discovery.
   */
  const PROVISIONERS_CREATE = [
    'RecurlyAegirSiteCloneProvisioner',
  ];

  /**
   * @var string $endpoint
   *   The Aegir service endpoint URL to send the request.
   */
  protected $endpoint;

  /**
   * @var string $hostname
   *   The virtual hostname of the site to provision.
   */
  protected $hostname;

  /**
   * @var string $template
   *   The machine name of the template to use in provisioning the site.
   */
  protected $template;

  /**
   * @var array $response
   *   The response data received after sending a provision request.
   */
  protected $response;

  /**
   * @var array $headers
   *   The response headers received after sending a provision request.
   */
  protected $headers;

  /**
   * @var integer $status
   *   The response code received after sending a provision request.
   */
  protected $status;

  /**
   * Class Constructor.
   *
   * @param string $endpoint
   *   The remote endpoint to send the provision request.
   * @param string $hostname
   *   The virtual hostname of the site to provision.
   * @param string $template
   *   The machine name of the template to use in provisioning the site.
   */
  public function __construct($endpoint, $hostname, $template = NULL) {

    // Ensure parameters are set.
    if (empty($endpoint)) {
      throw new Exception('Provisioners must be provided with an endpoint on construction.');
    }
    if (empty($hostname)) {
      throw new Exception('Provisioners must be provided with a hostname on construction.');
    }

    // Assign them to class variables.
    $this->endpoint = $endpoint;
    $this->hostname = $hostname;
    $this->template = $template;
  }

  /**
   * Gets the name of the task type used to provision a new site.
   *
   * @return string
   *   The name of the task type.
   */
  abstract public static function getTaskType();

  /**
   * Sends a request to Aegir to provision a site.
   *
   * Implementing classes must do the following:
   *   1. Make a service call to Aegir that creates a task to provision a site.
   *   2. Set $this->$task_id to the task ID returned in the call (if possible).
   *   3. Return $this for method chaining.
   *
   * @return $this
   *   The object itself.
   */
  abstract public function provision();

  /**
   * Sends a request to the Aegir host, and sets class variables with results.
   *
   * @param array $data
   *   The data to send.
   */
  protected function sendRequestAndReceiveResponse($data) {
    $client = \Drupal::httpClient();

    try {
      $request = $client->post($this->endpoint, ['json' => $data]);
    }
    catch (Exception $e) {
      watchdog_exception('recurly_aegir', $e->getMessage());
    }

    // Set request results.
    $this->status = $request->getStatusCode();
    $this->headers = $request->getHeaders();
    $this->response = json_decode((string) $request->getBody(), TRUE);
  }

  /**
   * Returns the Aegir task ID returned after provisioning a site.
   *
   * @return integer
   *   The task ID.
   * @throws Exception
   *   If there is no task ID because the service call failed.
   * @see $this->provision().
   */
  public function getTaskId() {
    if (empty($this->response['nid'])) {
      throw new RecurlyAegirTaskCreationFailedException(sprintf(
        'Task creation failed for %s from %s via %s.',
        $this->hostname,
        $this->template,
        (new \ReflectionClass(get_class($this)))->getShortName()
      ));
    }
    return $this->response['nid'];
  }

  /**
   * Returns the service enpoint associated with this provisioner.
   */
  public function getEndpoint() {
    return $this->endpoint;
  }

  /**
   * Returns the target hostname associated with this provisioner.
   */
  public function getHostname() {
    return $this->hostname;
  }

  /**
   * Returns the template used for provisioning new sites.
   */
  public function getTemplate() {
    return $this->template;
  }
}
